#!/usr/bin/env python

import argparse
from pathlib import Path
import numpy as np


def main(dataset_root, scene_name):
    path = dataset_root / 'scenes' / scene_name
    undistorted_path = path / 'rgb_undistort'
    distorted_path = path / 'distorted_images'
    rgbd_associations_path = path / 'orb_slam_rgbd_associations.txt'
    rgb_associations_path = path / 'orb_slam_rgb_associations.txt'
    depth_associations_path = path / 'orb_slam_depth_associations.txt'

    def get_filename_index(filename):
        return int(str(filename).split('_')[-1].split('.')[0])

    rgb_images = list(undistorted_path.glob('*.png'))
    rgb_images = sorted(rgb_images, key=get_filename_index)
    depth_images = [
        distorted_path /
            ('depth_' + str(get_filename_index(rgb_image)) + '.png')
        for rgb_image in rgb_images
    ]
    rgbd_images = list(zip(rgb_images, depth_images))

    rgbd_associations = [
        [
            str('%.1f' % get_filename_index(rgb_image)),
            str(rgb_image.parents[0]).split('/')[-1] + '/' + rgb_image.name,
            str('%.1f' % get_filename_index(depth_image)),
            str(depth_image.parents[0]).split('/')[-1] + '/' + depth_image.name,
        ]
        for i, (rgb_image, depth_image) in enumerate(rgbd_images)
        if rgb_image.is_file() and depth_image.is_file()
    ]
    rgb_associations = [i[:2] for i in rgbd_associations]
    depth_associations = [i[2:] for i in rgbd_associations]

    np.savetxt(str(rgbd_associations_path), rgbd_associations, fmt='%s')
    np.savetxt(str(rgb_associations_path), rgb_associations, fmt='%s')
    np.savetxt(str(depth_associations_path), depth_associations, fmt='%s')


if __name__ == '__main__':
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '--dataset-root', type=Path,
        default='/root/3d_dataset'
    )
    parser.add_argument(
        '--scene-name', type=str,
        default='scene2'
    )
    args = parser.parse_args()
    main(**args.__dict__)
