#!/usr/bin/env python


import argparse
from pathlib import Path
import numpy as np
from tqdm import tqdm
from pyquaternion import Quaternion


def main(dataset_root, scene_name):
    path = dataset_root / 'scenes' / scene_name
    orb_poses_path = path / 'CameraTrajectory.txt'
    if not orb_poses_path.is_file():
        sys.exit('ORB poses not found in %s' % orb_poses_path)
    orb_poses = np.loadtxt(orb_poses_path)

    def xyz_quat_to_mat(x, y, z, qx, qy, qz, qw):
        mat = Quaternion(qw, qx, qy, qz).transformation_matrix
        mat[:3, -1] = [x, y, z]
        return mat

    poses = {
        int(pose[0]): xyz_quat_to_mat(*(pose[1:].tolist()))
        for pose in orb_poses
    }
    np.save(path / 'camera_poses.npy', poses)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '--dataset-root', type=Path,
        default='/root/3d_dataset'
    )
    parser.add_argument(
        '--scene-name', type=str,
        default='scene2'
    )

    args = parser.parse_args()
    main(**args.__dict__)
