#!/usr/bin/env bash

SCENE_NAME=$1
DATASET_ROOT=/dataset

# undistort images
cd /toc_annotator
python3 undistort_images.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME" \
    --resize-factor 1

# generate intermediate files
cd /toc_annotator/dense_reconstruction
python3 generate_orb_slam_associations.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME"
python3 npy_params_to_orb_slam.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME" \
    --resize-factor 1

# run ORB_SLAM2
cd /opt/ORB_SLAM2
./Examples/RGB-D/rgbd_tum Vocabulary/ORBvoc.txt \
    "$DATASET_ROOT/scenes/$SCENE_NAME/orb_slam_params.yaml" \
    "$DATASET_ROOT/scenes/$SCENE_NAME" \
    "$DATASET_ROOT/scenes/$SCENE_NAME/orb_slam_rgbd_associations.txt"
mv KeyFrameTrajectory.txt "$DATASET_ROOT/scenes/$SCENE_NAME"
mv CameraTrajectory.txt "$DATASET_ROOT/scenes/$SCENE_NAME"

# generate camera poses npy
cd /toc_annotator/dense_reconstruction
python3 orb_slam_pose_to_npy.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME"

# generate intermediate files
python /opt/fastfusion/associate.py \
    "$DATASET_ROOT/scenes/$SCENE_NAME/CameraTrajectory.txt" \
    "$DATASET_ROOT/scenes/$SCENE_NAME/orb_slam_depth_associations.txt" \
    > /tmp/tmp.txt
python /opt/fastfusion/associate.py \
    /tmp/tmp.txt \
    "$DATASET_ROOT/scenes/$SCENE_NAME/orb_slam_rgb_associations.txt" \
    > "$DATASET_ROOT/scenes/$SCENE_NAME/fastfusion_associations.txt"

# run fastfusion
cd /opt/fastfusion
./bin/onlinefusion \
    "$DATASET_ROOT/scenes/$SCENE_NAME/fastfusion_associations.txt" \
    --thread-fusion --imagescale 1000.0 \
    --intrinsics "$DATASET_ROOT/scenes/$SCENE_NAME/intrinsics.txt"
mv current.ply "$DATASET_ROOT/scenes/$SCENE_NAME/fastfusion_reconstruction.ply"
