#!/usr/bin/env python
import sys
import logging
import datetime
import time
import argparse
import os
from pathlib import Path
import subprocess

def init_logger():
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    fh = logging.FileHandler('../dataset/mask_gen_{}.txt'.format(str(datetime.datetime.now())))
    fh.setFormatter(logging.Formatter('%(asctime)-15s -%(levelname)s - %(message)s'))
    fh.setLevel(logging.INFO)

    logger.addHandler(fh)

    return logger

logger = init_logger()


def run_mask_gen(dataset_root, scene_name, filter_labels):
    cmd = ['bash', 'generate_single_mask.sh', str(dataset_root), scene_name]
    cmd.extend(filter_labels)
    logger.info('running: ' + ' '.join(cmd))
    ret = subprocess.call(cmd)
    if ret != 0:
        logger.error('Problem with' + ' '.join(cmd))



if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--dataset-root', type=Path
    )

    parser.add_argument(
        '--filter-labels', type=str, nargs='+'
    )

    args = parser.parse_args()
    dataset_root = args.dataset_root
    filter_labels = args.filter_labels

    scenes_root = dataset_root/'scenes'
    scene_list_path = dataset_root/'scene_name_list.txt'
    with scene_list_path.open('r') as scene_list_file:
        scene_list = scene_list_file.read().splitlines()
        scene_list_file.close()
    scene_list.sort()

    logger.info('starting the batch masks generation')
    for scene_name in scene_list:
        if(scenes_root/scene_name).exists() and (not (scenes_root/scene_name/'mask_rcnn_masks').exists() or
            len(list((scenes_root/scene_name/'mask_rcnn_masks').iterdir())) == 0 ):
            run_mask_gen(dataset_root, scene_name, filter_labels)









