import os
class single_camera_integrity:
    def __init__(self,single_camera_integrity_list):
        self.rgb_integrity = single_camera_integrity_list[0]
        self.depth_integrity = single_camera_integrity_list[1]
        self.annotations_integrity = single_camera_integrity_list[2]
        self.masks_integrity = single_camera_integrity_list[3]
        self.rendered_image_integrity = single_camera_integrity_list[4]
        self.integrity_level = get_single_camera_integrity_level(single_camera_integrity_list)
    
    def get_integrity_level(self):
        return self.integrity_level

class scene_integrity:
    def __init__(self,integrity_list):
        self.integrity_list = integrity_list
        self.integrity_level = get_scene_integrity_level(self.integrity_list)
        self.rs_integrity = single_camera_integrity(self.integrity_list[1:6])
        self.kn_integrity = single_camera_integrity(self.integrity_list[6:])
        self.object_id_list_integrity = integrity_list[0]

    def get_object_id_list_integrity(self):
        return self.object_id_list_integrity
    
    def get_rs_integrity(self):
        return self.rs_integrity

    def get_kn_integrity(self):
        return self.kn_integrity

    def get_integrity_level(self):
        return self.integrity_level

def get_single_camera_integrity_level(single_camera_integrity_list):
    '''
    Input:
    single_camera_integrity_list:
    a list of single camera integrity:[rgb,depth,annotations,masks,rendered_images]
    
    Output:
    an integer, 
        0 for no integrity,
        1 for the original data has integrity,
        2 for the annotations exists,
        3 for masks exists,
        4 for rendered image exists
    '''
    single_camera_integrity_level = 0
    base = 0
    if single_camera_integrity_list[base] and single_camera_integrity_list[base + 1]:
        single_camera_integrity_level = 1
    for i in range(3):
        if single_camera_integrity_list[i + base + 2] is True:
            single_camera_integrity_level += 1
        else:
            break
    return single_camera_integrity_level

def get_scene_integrity_level(integrity_list):
    '''
    Input:
    integrity_list: the list of 11 elements defined in the function get_scene_integrity_list
    
    Output:
    List of 3 integers
    [object_id_list,realsense,kinect]
    for object_id_list, 
        0 for no presence,
        1 for presence
    for realsense and kinect:
        0 for no integrity,
        1 for the original data has integrity,
        2 for the annotations exists,
        3 for masks exists,
        4 for rendered image exists
    '''
    object_id_list = int(integrity_list[0])
    realsense_level = get_single_camera_integrity_level(integrity_list[1:6])
    kinect_level = get_single_camera_integrity_level(integrity_list[6:])
    return [object_id_list,realsense_level,kinect_level]

def get_scene_integrity(scene_path):
    '''
    Input:
    dataset_dir:the directory for the whole dataset
    scene:the directory name for the scene which we want to check

    Output:
    An scene_integrity class instance
    '''
    integrity_list = get_scene_integrity_list(scene_path)
    return scene_integrity(integrity_list)

def get_scene_integrity_list(scene_path):
    '''
    Input:
    dataset_dir:the directory for the whole dataset
    scene:the directory name for the scene which we want to check

    Output:
    A list of boolean variables, True for integrity and False for no integrity.
    [object_id_list,rs_rgb,rs_depth,rs_annotations,rs_masks,rs_rendered_images,kn_rgb,kn_depth,kn_annotations,kn_masks,kn_rendered_images]
    '''
    if not os.path.exists(scene_path):
        raise ValueError('scene directory %s doesn\'t exist' % scene_path)
    integrity = [True for i in range(11)]
    
    dirs = ['rgb','depth','annotations','masks','rendered_image']
    postfix=['.png','.png','.xml','.npy','.png']
    # check object_id_list.txt
    if not os.path.exists(os.path.join(scene_path,'object_id_list.txt')):
        integrity[0] = False
    # check data for realsense
    base = 1
    root_dir = os.path.join(scene_path,'data')
    for i in range(5):
        integrity[i + base]=check_dir(256,os.path.join(root_dir,dirs[i]),postfix[i])
    
    # check data for kinect
    base = 6
    root_dir = os.path.join(scene_path,'data_kinect')
    for i in range(5):
        integrity[i + base]=check_dir(256,os.path.join(root_dir,dirs[i]),postfix[i])
    return integrity

def check_dir(number,dir_path,postfix):
    '''
    Input:
    number: the number of files that should be in the directory
    dir_path: the path of the checked directory
    postfix: the postfix of files.
        ########## THE DOT SHOULD BE INCLUDED ##########
        '.png' for example

    Output:
    A boolean variable, True for integrity and False for no integrity.
    '''
    if not os.path.exists(dir_path):
        print('[INTEGRITY CHECK]:path %s doesn\'t exist' % dir_path)
        return False
    flag = True
    for i in range(number):
        if not os.path.exists(os.path.join(dir_path,('%04d' % i)+postfix)):
            flag = False
            print('[INTEGRITY CHECK]:file %s doesn\'t exist' % os.path.join(dir_path,('%04d' % i)+postfix))
    return flag