import os
import open3d as o3d
import cv2
import numpy as np
import scipy 
import matplotlib.pyplot as plt
from trans3d import *
import shutil
import copy
def get_pinhole_camera_parameters(camera):
    pinhole_cam_params = o3d.camera.PinholeCameraParameters()
    pinhole_cam_params.extrinsic = np.eye(4,dtype=np.float64)
    if camera == 'realsense':
        pinhole_cam_params.intrinsic= o3d.camera.PinholeCameraIntrinsic(width=1280,height=720,fx=927.17,fy=927.37,cx=651.32,cy=349.62)
    elif camera == 'kinect':
        pinhole_cam_params.intrinsic= o3d.camera.PinholeCameraIntrinsic(width=1280,height=720,fx=631.55,fy=631.21,cx=638.44,cy=366.50)
    else:
        raise ValueError('Cannot recognize camera type:%s' % camera)
    return pinhole_cam_params

def get_mask(model_pcd_list,camera,pose_list):
    """
    Input:
    model_pcd_list: a list of point cloud created by open3d
    camera: 'realsense' or 'kinect'
    pose_list: the pose vector list contains the geometry and object id. The definition is in xmlhandller.py

    Output:
    numpy array of the mask of object where the value is the id of the object
    """
    # print('log:begin calculate mask for pose list:'+pose_list.__repr__())
    # camera_params = get_pinhole_camera_parameters(camera)
    if camera == 'realsense':
        #camera_params = o3d.io.read_pinhole_camera_parameters('viewpoint_realsense.json')
        camera_params = o3d.io.read_pinhole_camera_parameters('viewpoint.json')
    elif camera == 'kinect':
        camera_params = o3d.io.read_pinhole_camera_parameters('viewpoint_kinect.json')
    else:
        raise ValueError('Cannot recognize camera:%s' % camera)
    visualizer = o3d.visualization.Visualizer()
    # visualizer.reset_view_point(False)
    visualizer.create_window(width = 1280,height = 720)
    visualizer_view_controller = visualizer.get_view_control()
    for pose in pose_list:
        pose_id = pose.get_id()
        # It is important to use copy.deepcopy or the pcd will be changed
        pcd = copy.deepcopy(model_pcd_list[pose_id])
        pcd_transformed = pcd.transform(pose.get_mat_4x4())
        pcd_transformed.paint_uniform_color([pose_id / 255,pose_id / 255,pose_id / 255])
        visualizer.add_geometry(pcd_transformed)
    visualizer_view_controller.convert_from_pinhole_camera_parameters(camera_params)
    visualizer.update_geometry()
    visualizer.poll_events()
    temp_int = 0
    while(os.path.exists('temp_%d.png' % temp_int)):
        temp_int += 1
    temp_png_file_name = 'temp_%d.png' % temp_int
    visualizer.capture_screen_image(temp_png_file_name,do_render = False)
    visualizer.destroy_window()
    mask = cv2.imread(temp_png_file_name,cv2.IMREAD_GRAYSCALE)
    assert mask is not None
    os.remove(temp_png_file_name)
    return mask
