from trans3d import *
from rgbd_image_to_pcd import rgbd_to_pointcloud
from argparse import ArgumentParser
import os
import open3d as o3d
from xmlhandler import *
from pose import *
import numpy as np

parser = ArgumentParser()
parser.add_argument('--data_dir',default='scene_33',help='The directory of the scene')
parser.add_argument('--model_dir',default='models',help='The directory of the ply models')
parser.add_argument('--object_file_name_list',default='object_file_name_list.txt',help='The txt file that specifies the names of the models')
parser.add_argument('--object_num',default=0,type=int,help='the number of the object in the object_id_list that want to be drawn in the image')
parser.add_argument('--frame_num',default=0,type=int,help='The number of frame that want to be shown in the image')
parser.add_argument('--camera', default='realsense',help='cameara type, realsense or kinect')
parser.add_argument('--xml_dir', default='annotations',help='output xml file directory')

cam_rs = np.array([927.17, 0.0, 651.32, 0.0, 927.37, 349.62, 0.0, 0.0, 1.0]).reshape((3, 3))
cam_kn = np.array([631.54864502,0.0,638.43517329,0.,631.20751953,366.49904066,0.0,0.0,1.0]).reshape((3, 3))

FLAGS = parser.parse_args()
if FLAGS.camera == 'realsense':
    CAMERA_TYPE_DIR = 'data'
    CAMERA_MATRIX=cam_rs
elif FLAGS.camera == 'kinect':
    CAMERA_TYPE_DIR = 'data_kinect'
    CAMERA_MATRIX=cam_kn
else:
    raise ValueError('Cannot recognize camera type:%s' % FLAGS.camera)

OBJECT_ID_LIST_FILE_NAME=os.path.join(FLAGS.data_dir,'object_id_list.txt')
OBJECT_FILE_NAME_LIST_FILE_NAME=FLAGS.object_file_name_list
FRAME_NUMBER = FLAGS.frame_num
OBJECT_NUMBER = FLAGS.object_num

ALL_DATA_DIR = os.path.join(FLAGS.data_dir, CAMERA_TYPE_DIR)
RGB_IMAGE_DIR = os.path.join(FLAGS.data_dir, CAMERA_TYPE_DIR, 'rgb')
DEPTH_IMAGE_DIR = os.path.join(FLAGS.data_dir, CAMERA_TYPE_DIR, 'depth')
XML_DIR = os.path.join(FLAGS.data_dir, CAMERA_TYPE_DIR, FLAGS.xml_dir)

object_file_name_list_file = open(OBJECT_FILE_NAME_LIST_FILE_NAME,'r')
lines = object_file_name_list_file.readlines()
objectfilenamelist = []
for line in lines:
    if not (line == '\n'):
        objectfilenamelist.append(line.replace('\n','').replace('\r',''))
print('log:loaded object file name list:',end='')
print(objectfilenamelist)

object_id_list_file = open(OBJECT_ID_LIST_FILE_NAME,'r')
lines = object_id_list_file.readlines()
objectidlist=[]
for line in lines:
    if not (line == '\n'):
        objectidlist.append(int(line))
print('log:loaded object id list:',end='')
print(objectidlist)

num_rgb_image = len(os.listdir(RGB_IMAGE_DIR))
num_depth_image = len(os.listdir(DEPTH_IMAGE_DIR))
assert num_depth_image == num_rgb_image
frame_number = num_depth_image
print('log:total frames number:%d' % frame_number)

rgb_image_path = os.path.join(RGB_IMAGE_DIR,'%04d.png' % FRAME_NUMBER)
depth_image_path = os.path.join(DEPTH_IMAGE_DIR,'%04d.png' % FRAME_NUMBER)
pcd = rgbd_to_pointcloud(rgb_image_path,depth_image_path,1280,720,CAMERA_MATRIX)

model = o3d.io.read_point_cloud(os.path.join(FLAGS.model_dir,objectfilenamelist[objectidlist[OBJECT_NUMBER]]))
main_xml_reader = xmlReader(os.path.join(XML_DIR,'%04d.xml' % FRAME_NUMBER))

pose_vector_list = main_xml_reader.getposevectorlist()
pose_vector = pose_vector_list[OBJECT_NUMBER]
pose = pose_from_pose_vector(pose_vector)

camera_pose = np.load('realsense_pose.npy')
camera_pose = camera_pose[FRAME_NUMBER]

##########################################################################################
##########################################################################################
##########################################################################################
############# uncomment to get param first and then comment to get picture ###############
##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
# vis=o3d.visualization.Visualizer()

# vis.create_window(width=1280,height = 720)
# vis.add_geometry(model.transform(pose.get_mat_4x4()))
# vis.add_geometry(pcd.transform(np.linalg.inv(camera_pose)))
# vis.run()
# param = vis.get_view_control().convert_to_pinhole_camera_parameters()
# o3d.io.write_pinhole_camera_parameters('param.json', param)
# vis.destroy_window()

vis=o3d.visualization.Visualizer()
vis.create_window(width=1280,height = 720)
ctr = vis.get_view_control()
param = o3d.io.read_pinhole_camera_parameters('param.json')
vis.add_geometry(pcd.transform(np.linalg.inv(camera_pose)).voxel_down_sample(0.005))
ctr.convert_from_pinhole_camera_parameters(param)
vis.capture_screen_image('1.png', do_render=True)
vis.run()
vis.destroy_window()

# vis=o3d.visualization.Visualizer()
# vis.create_window(width=1280,height = 720)
# ctr = vis.get_view_control()
# param = o3d.io.read_pinhole_camera_parameters('param.json')
# vis.add_geometry(model.transform(pose.get_mat_4x4()))
# vis.add_geometry(pcd.transform(np.linalg.inv(camera_pose)).voxel_down_sample(0.005))
# ctr.convert_from_pinhole_camera_parameters(param)
# vis.capture_screen_image('2.png', do_render=True)
# vis.run()
# vis.destroy_window()