# untested
from get_mask import get_mask
from renderer import draw_model
import copy
import cv2
import numpy as np
def mask_paste(background_image,mask,pose_id,pre_rendered_image):
    """
    Input:
    background_image: the background image that want to paste object on
    mask: the numpy array mask
    pose_id: the is of this object that want to paste on the image
    pre_rendered_image: the rendered object is on this image

    Output:
    The image that the object is rendered preperly using the mask
    """
    assert background_image.shape[:2] == mask.shape[:2]
    mask_bool = (mask == pose_id)
    mask_int = mask_bool.astype(int)
    reverse_mask_int = 1 - mask_int
    reverse_mask_int_3_channel = np.empty((reverse_mask_int.shape[0],reverse_mask_int.shape[1],3))
    mask_int_3_channel = np.empty((mask_int.shape[0],mask_int.shape[1],3))
    for i in range(3):
        reverse_mask_int_3_channel[:,:,i] = reverse_mask_int
        mask_int_3_channel[:,:,i] = mask_int
    return background_image * reverse_mask_int_3_channel + mask_int_3_channel * pre_rendered_image 
    

def render_image(model_list,model_pcd_list,background_image,camera,pose_list):
    """
    Input:
    model_list: a list of models which are the instances of Model class in model.py
    model_pcd_list: a list of point cloud created by open3d
    camera: 'realsense' or 'kinect'
    pose_list: the pose vector list contains the geometry and object id. The definition is in xmlhandller.py
    
    Output:
    numpy array of the mask of object where the value is the id of the object
    """
    if camera == 'realsense':
        camera_intrinsic_matrix = np.array([927.17, 0.0, 651.32, 0.0, 927.37, 349.62, 0.0, 0.0, 1.0]).reshape((3, 3))
    elif camera == 'kinect':
        camera_intrinsic_matrix = np.array([631.54864502,0.0,638.43517329,0.,631.20751953,366.49904066,0.0,0.0,1.0]).reshape((3, 3))
    else:
        raise ValueError('Can not recognize camera type of %s.' % camera)
    mask = get_mask(model_pcd_list,camera,pose_list)
    # bg_image = copy.deepcopy(background_image)
    rendered_image = copy.deepcopy(background_image)
    for pose in pose_list:
        pose_id = pose.get_id()
        print('log:render for model id:%d' % pose_id)
        # it is neccessary to combine pre_rendered_image and mask to get the right image
        pre_rendered_image = draw_model(background_image,pose.get_mat_4x4(),camera_intrinsic_matrix,model_list[pose_id])
        rendered_image=mask_paste(rendered_image,mask,pose_id,pre_rendered_image)
    return rendered_image