import argparse
import os, sys
from transforms3d.quaternions import quat2mat
import open3d as o3d
from open3d import *
import scipy.io as sio
import cv2
import numpy as np
from random import *
from scipy.stats import special_ortho_group
import matplotlib.pyplot as plt
import matplotlib
import scipy
import time

if __name__ == '__main__':
    num_images = 50000
    height = 480
    width = 640
    fx = 1066.778
    fy = 1066.778
    px = 319.5
    py = 239.5
    factor_depth = 10000.0
    seed = 2000
    intrinsic_matrix = np.array([[fx, 0, px], [0, fy, py], [0, 0, 1]])
    root = './data/original/data_syn/'
    vis = o3d.visualization.Visualizer()
    vis.create_window(width = 640, height = 480)
    ctr = vis.get_view_control()
    param = o3d.io.read_pinhole_camera_parameters('viewpoint.json')
    for i in range(num_images):
        pcd_news = [None]*10
        pcd_lab = [None]*10
        num = randrange(2,5)
        qt = np.zeros((3, 4, num), dtype=np.float32)
        no = sample(range(10),num)
        for j in range(num):
            pcd = o3d.io.read_point_cloud('./data/YCB-Video/plys/'+str(no[j]+1)+'.ply')
            rot = special_ortho_group.rvs(3)
            x = (2*random()-1)/5
            y = (2*random()-1)/5
            z = (2*random()-1)/5
            loc = np.array([x,y,z]).reshape(-1,1)
            rt = np.hstack([rot,loc])
            qt[:,:,j] = rt
            trans = np.vstack([rt,[0,0,0,1]])
            pcd_news[no[j]] = pcd.transform(trans)
            vis.add_geometry(pcd_news[no[j]])
        ctr.convert_from_pinhole_camera_parameters(param)
        vis.update_geometry()
        vis.poll_events()
        filename = root + '{:06d}-color.png'.format(i)
        vis.capture_screen_image(filename,do_render=True)
        filename = root + '{:06d}-depth.png'.format(i)
        vis.capture_depth_image(filename,do_render=True)
        for j in range(num):
            vis.remove_geometry(pcd_news[no[j]])
        for j in range(num):
            pcd = o3d.io.read_point_cloud('./data/YCB-Video/plys/'+str(no[j]+1)+'.ply')
            rt = qt[:,:,j]
            trans = np.vstack([rt,[0,0,0,1]])
            pcd_lab[no[j]] = pcd.transform(trans)
            pcd_lab[no[j]].paint_uniform_color([(no[j]+1)/255,(no[j]+1)/255,(no[j]+1)/255])
            vis.add_geometry(pcd_lab[no[j]])
        ctr.convert_from_pinhole_camera_parameters(param)
        vis.update_geometry()
        vis.poll_events()
        filename = root + '{:06d}-label.png'.format(i)
        vis.capture_screen_image(filename,do_render=True)
        for j in range(num):
            vis.remove_geometry(pcd_lab[no[j]])
        #metadata
        metadata = {'poses': qt,'cls_indexes':np.asarray(no)+1,\
        'intrinsic_matrix': intrinsic_matrix, 'factor_depth': factor_depth}
        filename = root + '{:06d}-meta.mat'.format(i)
        scipy.io.savemat(filename, metadata, do_compression=True)  
    vis.destroy_window()

        




