import copy
from get_mask import get_mask
from unpack import unpack
from xmlhandler import xmlReader
import argparse
from pose import pose_from_pose_vector, pose_list_from_pose_vector_list
import os
import numpy as np
from xmlhandler import getframeposevectorlist
import open3d as o3d
from render_image import render_image
from model import loadmodel
import cv2
import shutil
parser = argparse.ArgumentParser()
parser.add_argument('--num_frame', default=0, type=int, help='number of frame')
parser.add_argument('--camera', default='realsense',
                    help='cameara type, realsense or kinect')
parser.add_argument('--image_name', default='6dpose-vis.png',
                    help='the file name for the output image')
parser.add_argument('--model_dir', default='models',
                    help='ply model files directory path')
parser.add_argument('--data_dir', default='scene_33',
                    help='scene directory name')
parser.add_argument('--object_id_list', default='object_id_list.txt',
                    help='object id list text file name')
parser.add_argument('--object_file_name_list', default='object_file_name_list.txt',
                    help='ascii text file name that specifies the filenames of all possible objects')
parser.add_argument('--xml_dir', default='annotations',
                    help='output xml file directory')
parser.add_argument('--tar_dir', default='directory of tar files',
                    help='directory where the tar files locate')

FLAGS = parser.parse_args()

if FLAGS.camera == 'realsense':
    CAMERA_TYPE_DIR = 'data'
elif FLAGS.camera == 'kinect':
    CAMERA_TYPE_DIR = 'data_kinect'
else:
    raise ValueError('Cannot recognize camera type:%s' % FLAGS.camera)
OBJECT_ID_LIST_FILE_NAME=os.path.join(FLAGS.data_dir,FLAGS.object_id_list)
OBJECT_FILE_NAME_LIST_FILE_NAME=FLAGS.object_file_name_list
TAR_DIR = FLAGS.tar_dir


ALL_DATA_DIR = os.path.join(FLAGS.data_dir, CAMERA_TYPE_DIR)
RGB_IMAGE_DIR = os.path.join(FLAGS.data_dir, CAMERA_TYPE_DIR, 'rgb')
DEPTH_IMAGE_DIR = os.path.join(FLAGS.data_dir, CAMERA_TYPE_DIR, 'depth')
XML_DIR = os.path.join(FLAGS.data_dir, CAMERA_TYPE_DIR, FLAGS.xml_dir)

def get_model_pcd_list(model_dir,model_name_list):
    """
    Input:
    model_dir:the directory where the models locate
    model_name_list: the list of model name that want to load

    Output:
    A list of the point cloud raed by open3d library
    """
    model_pcd_list=[]
    print('log:loading ply models using open3d')
    for model_name in model_name_list:
        model_pcd_list.append(o3d.io.read_point_cloud(os.path.join(model_dir,model_name)))
    print('log:ply models loaded')
    return model_pcd_list

def save_view_point(pcd, filename):
    vis = o3d.visualization.Visualizer()
    vis.create_window(width=1280, height=720)
    vis.add_geometry(pcd)
    vis.run()  # user changes the view and press "q" to terminate
    param = vis.get_view_control().convert_to_pinhole_camera_parameters()
    o3d.io.write_pinhole_camera_parameters(filename, param)
    vis.destroy_window()


def load_view_point(pcd, filename, image_name):
    vis = o3d.visualization.Visualizer()
    vis.create_window(width=1280, height=720)
    ctr = vis.get_view_control()
    param = o3d.io.read_pinhole_camera_parameters(filename)
    vis.add_geometry(pcd)
    ctr.convert_from_pinhole_camera_parameters(param)
    vis.capture_screen_image(image_name, do_render=True)
    vis.destroy_window()


def merge_pointclouds(pcd1, pcd2):
    merged_points = np.vstack(
        (np.asarray(pcd1.points), np.asarray(pcd2.points)))
    merged_colors = np.vstack(
        (np.asarray(pcd1.colors), np.asarray(pcd2.colors)))

    merged_pcd = o3d.geometry.PointCloud()
    merged_pcd.points = o3d.utility.Vector3dVector(merged_points)
    merged_pcd.colors = o3d.utility.Vector3dVector(merged_colors)

    return merged_pcd


def vis_6dpose(model_pcd_list, pose_list, picture_file_name):
    '''
    Input:
    model_pcd_list:a list of point cloud read by open3d
    pose_list: a list of pose class instance defined in pose.py

    Output:
    True for success
    False for failure

    Function:
    generate a picture that visualize the 6d poses of objects
    '''
    # model_pose = np.load('poses.npy')
    # print('model_pose')
    # print(model_pose)
    # model_pcd= o3d.io.read_point_cloud("models/box.ply")
    # # model_pcd.transform(np.linalg.inv(model_pose))
    # model_pcd.transform(model_pose)
    # full_pcd = model_pcd
    # full_pcd = merge_pointclouds(model_pcd,full_pcd)
    # print(len(pcds))
    # merge
    pcds = []
    for pose in pose_list:
        pcd = copy.deepcopy(model_pcd_list[pose.get_id()])
        pcds.append(pcd.transform(pose.get_mat_4x4()))

    fullpcd = pcds[0]
    for i in range(1, len(pcds)):
        fullpcd = merge_pointclouds(fullpcd, pcds[i])

    temp_int = 0
    while(os.path.exists('temp_%d.json' % temp_int)):
        temp_int += 1
    temp_json_file_name = 'temp_%d.json' % temp_int
    save_view_point(fullpcd, temp_json_file_name)
    load_view_point(fullpcd, temp_json_file_name,picture_file_name)
    os.remove(temp_json_file_name)

def main():
    object_file_name_list_file = open(OBJECT_FILE_NAME_LIST_FILE_NAME,'r')
    lines = object_file_name_list_file.readlines()
    objectfilenamelist = []
    for line in lines:
        if not (line == '\n'):
            objectfilenamelist.append(line.replace('\n','').replace('\r',''))
    print('log:loaded object file name list:',end='')
    print(objectfilenamelist)

    object_id_list_file = open(OBJECT_ID_LIST_FILE_NAME,'r')
    lines = object_id_list_file.readlines()
    objectidlist=[]
    for line in lines:
        if not (line == '\n'):
            objectidlist.append(int(line))
    print('log:loaded object id list:',end='')
    print(objectidlist)


    num_rgb_image = len(os.listdir(RGB_IMAGE_DIR))
    num_depth_image = len(os.listdir(DEPTH_IMAGE_DIR))
    assert num_depth_image == num_rgb_image
    frame_number = num_depth_image
    print('log:total frames number:%d' % frame_number)
    if not os.path.exists(XML_DIR):
        unpack(scene_name = FLAGS.data_dir,tar_file_directory = TAR_DIR,scene_directory = '.',force_move = True)
    frameposevectorlist = getframeposevectorlist(objectidlist,True,frame_number,XML_DIR)
    assert frame_number == len(frameposevectorlist)
    pose_list =pose_list_from_pose_vector_list(frameposevectorlist[FLAGS.num_frame])
    model_pcd_list = get_model_pcd_list(FLAGS.model_dir,objectfilenamelist)
    
    vis_6dpose(model_pcd_list,pose_list,FLAGS.image_name)

if __name__ == '__main__':
    main()
