from get_mask import get_mask
import cv2
import numpy as np
import matplotlib.pyplot as ply
import os
import argparse
import random
from check_integrity import get_scene_integrity
import shutil
parser = argparse.ArgumentParser()
parser.add_argument('--data_dir', default='scene_33',
                    help='data directory name')
parser.add_argument('--num_frame', default=0, type=int, help='number of frame')
parser.add_argument('--transparency', default=0.2,
                    type=float, help='the mask transparency')
parser.add_argument('--camera', default='realsense',
                    help='cameara type, realsense or kinect')
parser.add_argument('--mask_dir', default='masks',
                    help='the directory where the mask npy files locate')
parser.add_argument('--object_id_list', default='object_id_list.txt',
                    help='object id list text file name')
FLAGS = parser.parse_args()

OBJECT_ID_LIST_FILE_NAME = os.path.join(FLAGS.data_dir, FLAGS.object_id_list)

if FLAGS.camera == 'realsense':
    CAMERA_TYPE_DIR = 'data'
elif FLAGS.camera == 'kinect':
    CAMERA_TYPE_DIR = 'data_kinect'
else:
    raise ValueError('Cannot recognize camera type:%s' % FLAGS.camera)

def add_scene_mask(scene_path,force_remove=True):
    '''
    Input:
    scene_path:the scene dierectory path

    Function:
    generating rendered mask for all pictures in this scene.
    '''
    integrity = get_scene_integrity(scene_path)
    integrity_level = integrity.get_integrity_level()
    if integrity_level[1] < 3 or integrity_level[2] < 3 or integrity_level[1] == 0:
        raise ValueError('scene integrity level too low:%s' %(integrity_level.__repr__()))
    if integrity_level[1] == 4 or integrity_level[2] == 4:
        if force_remove == False:
            raise ValueError('rendered images exist and force is set False')
    shutil.rmtree(os.path.join(scene_path,'data','rendered_image'))
    shutil.rmtree(os.path.join(scene_path,'data_kinect','rendered_image'))

    os.mkdir(os.path.join(scene_path,'data','rendered_image'))
    os.mkdir(os.path.join(scene_path,'data_kinect','rendered_image'))

    object_id_list_file = open(os.path.join(scene_path,'object_id_list.txt'), 'r')
    lines = object_id_list_file.readlines()
    objectidlist = []
    for line in lines:
        if not (line == '\n'):
            objectidlist.append(int(line))
    print('log:loaded object id list:', end='')
    print(objectidlist)
    
    print('log:generating rendered image for %s' % scene_path)
    for i in range(256):
        print('\r%03d/256' % (i + 1),end='')
        for dir in ['data','data_kinect']:
            bg_image = cv2.imread(os.path.join(scene_path,dir,'rgb','%04d.png' % i))
            mask = np.load(os.path.join(scene_path,dir,'masks','%04d.npy' % i))
            image_mask = add_mask(bg_image,mask,objectidlist)
            cv2.imwrite(os.path.join(scene_path,dir,'rendered_image','%04d.png' % i), image_mask)
    print('')

def add_mask(background_image, mask, id_list):
    '''
    Input:
    background_image: the image that want add mask on
    mask: the mask numpy array, with background value of 255
    id_list: the list of ids that want to show in this image

    Output:
    image has mask on it
    '''

    render_image = background_image
    for id in id_list:
        mask_float = (mask == id).astype(float)
        reverse_mask_float = 1 - mask_float

        mask_3_channel = np.empty((mask_float.shape[0], mask_float.shape[1], 3))
        reverse_mask_3_channel = np.empty((mask_float.shape[0], mask_float.shape[1], 3))
        single_color_image = np.empty((mask_float.shape[0], mask_float.shape[1], 3))
        for i in range(3):
            single_color_image[:,:,i] = np.ones(shape=mask_float.shape) * random.randint(100,200)
            mask_3_channel[:,:,i] = mask_float
            reverse_mask_3_channel[:,:,i] = reverse_mask_float
        render_image = render_image * (reverse_mask_3_channel + FLAGS.transparency *
                                       mask_3_channel) + single_color_image * (1-FLAGS.transparency) * mask_3_channel
    return render_image.astype(np.uint8)


def main():
    """
    Function:
    The main function that generates the mask for a image.
    """
    object_id_list_file = open(OBJECT_ID_LIST_FILE_NAME, 'r')
    lines = object_id_list_file.readlines()
    objectidlist = []
    for line in lines:
        if not (line == '\n'):
            objectidlist.append(int(line))
    print('log:loaded object id list:', end='')
    print(objectidlist)

    background_image = cv2.imread(os.path.join(
        FLAGS.data_dir, CAMERA_TYPE_DIR, 'rgb', '%04d.png' % FLAGS.num_frame))
    mask = np.load(os.path.join(FLAGS.data_dir, CAMERA_TYPE_DIR,
                                'masks', '%04d.npy' % FLAGS.num_frame))
    mask_image = add_mask(background_image, mask, objectidlist)
    cv2.imshow('mask image', mask_image)
    cv2.waitKey(0)
    cv2.destroyAllWindows()


if __name__ == '__main__':
    main()
