#! usr/bin/env python3

from pathlib import Path
from tqdm import tqdm
import pymeshlab
import os
import argparse


parser = argparse.ArgumentParser()

parser.add_argument(
    '--dataset-root', type=Path,
)

args = parser.parse_args()
args_dict = args.__dict__
dataset_root = args_dict['dataset_root']

model_list_path = Path('../dataset/object_name_list.txt')
with model_list_path.open('r') as model_list_file:
    model_list = model_list_file.read().splitlines()
    model_list_file.close()

ms = pymeshlab.MeshSet()
with model_list_path.open('a') as model_list_file:
    for model in tqdm(list(dataset_root.glob('*/*.obj'))):
        name = model.parent.name
        if model_list.count(name) == 0:
            ms.load_new_mesh(model.as_posix())
            for file_to_delete in list(model.parent.glob('*')):
                os.remove(file_to_delete.as_posix())
            ms.transform_scale_normalize(axisx=0.001, axisy=0.001, axisz=0.001)
            ms.save_current_mesh((model.parent/'textured.obj').as_posix())
            ms.transfer_color_texture_to_vertex()
            ms.save_current_mesh((model.parent/'visual.ply').as_posix(), save_vertex_flag=True, save_vertex_normal=True, save_face_flag=True)
            print('\033[032m{} processing finished. \033[0m'.format(name))
            model_list_file.write('{}\n'.format(name))
        else:
            print('\033[032m{} alreay processed before. \033[0m'.format(name))

