#!/usr/bin/env bash

DATASET_ROOT=$1
SCENE_NAME=$2
RESIZE_FACTOR=2

# undistort images
python undistort_images.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME" \
    --resize-factor "$RESIZE_FACTOR"
retVal=$?
if [ $retVal -ne 0 ]; then
    echo "Error in undistort"
    exit $retVal
fi

# render objects at random pose
ulimit -Sn 10000
python renderer.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME" \
    --resize-factor "$RESIZE_FACTOR"
retVal=$?
if [ $retVal -ne 0 ]; then
    echo "Error in renderer"
    exit $retVal
fi

# Run COLMAP sfm
python camera_pose_estimation.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME" \
    --resize-factor "$RESIZE_FACTOR"
retVal=$?
if [ $retVal -ne 0 ]; then
    echo "Error in sfm"
    exit $retVal
fi

# Run object pose estimation
python pose_estimation.py \
    --dataset-root "$DATASET_ROOT" --scene-name "$SCENE_NAME" \
    --resize-factor "$RESIZE_FACTOR"
retVal=$?
if [ $retVal -ne 0 ]; then
    echo "Error in pose_estimation"
    exit $retVal
fi
