#!/usr/bin/env python
import sys
sys.path.append('dataset_toolkit')


from toc_dataset_toolkit import TOC_Dataset
from toc_dataset_toolkit.utils.logging import set_log_level
from renderer import resize_camera_parameters
from pose_estimation import viz_objects

from align_utils import pose_align
from constants import get_workspace_bounding_box

import argparse
from pathlib import Path
import itertools
import time
import subprocess
import multiprocessing
import logging
import shutil
import re
import copy
import tempfile
import tqdm

import numpy as np
from scipy.spatial.transform import Rotation
import open3d as o3d
import pyrender
import trimesh
import matplotlib.pyplot as plt
import cv2


def main(dataset_root, scene_name, resize_factor):
    set_log_level('WARNING')
    dataset = TOC_Dataset(root=dataset_root)
    scene_name_list = dataset.load_scene_name_list()
    scene_id = scene_name_list.index(scene_name)

    print('scene', scene_id, scene_name)
    scene_dir = dataset_root / 'scenes' / scene_name_list[scene_id]

    camera_poses = np.load(
        str(scene_dir / 'camera_poses.npy'),
        allow_pickle=True
    ).item()
    object_poses = np.load(
        str(scene_dir / 'object_poses.npy'),
        allow_pickle=True
    ).item()

    viz_objects(
        dataset, scene_id, object_poses, resize_factor,
        camera_poses, None, None
    )


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--dataset-root', type=Path,
        default='/root/3d_dataset'
    )
    parser.add_argument(
        '--scene-name', type=str,
        default='scene2'
    )
    parser.add_argument('--resize-factor', type=int, default=2)
    args = parser.parse_args()
    main(**args.__dict__)
